'use strict';

var reactNative = require('react-native');
var React = require('react');

/**
 * This set of utilities + context allows React Native components to inherit
 * styles in the same way CSS does.
 */
/**
 * This should be the intersection of CSS properties defined
 * as inherited and React Native text styles that are
 * defined on the `TextStyle` type.
 *
 * @see https://www.sitepoint.com/css-inheritance-introduction/
 */
const inheritedStyles = [
    'color',
    'direction',
    'fontFamily',
    'fontSize',
    'fontStyle',
    'fontVariant',
    'fontWeight',
    'letterSpacing',
    'lineHeight',
    'textAlign',
    'textDecorationColor',
    'textShadowColor',
    'textShadowOffset',
    'textShadowRadius',
    'textTransform'
];
/**
 * For now, the only root style / property guaranteed is the `color` prop
 */
const rootStyles = {
    color: 'black'
};
const StyleContext = React.createContext(rootStyles);
const useStyles = (style) => {
    const inherited = React.useContext(StyleContext);
    if (!style) {
        return [inherited, undefined];
    }
    let newContext;
    const newStyles = Object.entries(reactNative.StyleSheet.flatten(style)).filter(([_, value]) => value !== undefined);
    const hasInheritedStyles = newStyles.find(([key]) => inheritedStyles.includes(key));
    if (hasInheritedStyles) {
        newContext = [];
    }
    const finalStyleEntries = Object.entries(inherited)
        .concat(newStyles);
    let finalStyle = Object.fromEntries(finalStyleEntries);
    finalStyle = Object.fromEntries(finalStyleEntries.map(([key, value]) => {
        if (typeof value === 'string') {
            value = value.replace(/currentColor/g, finalStyle.color);
        }
        const finalValue = [key, value];
        if (hasInheritedStyles &&
            inheritedStyles.includes(key)) {
            newContext.push(finalValue);
        }
        return finalValue;
    }));
    return [
        finalStyle,
        hasInheritedStyles
            ? Object.fromEntries(newContext)
            : undefined
    ];
};

const wrapContext = (Component) => {
    if (!Component) {
        return undefined;
    }
    return ({ style, ...props }) => {
        const [finalStyle, newContext] = useStyles(style);
        // @ts-ignore - not sure how to properly type this
        const JSX = React.createElement(Component, { style: finalStyle, ...props });
        if (newContext) {
            return (React.createElement(StyleContext.Provider, { value: newContext }, JSX));
        }
        return JSX;
    };
};

/**
 * These are asserted as not `undefined` for testing,
 * but they may be undefined at runtime if the underlying
 * React Native component is undefined at export time.
 */
const FlatList = wrapContext(reactNative.FlatList);
const ImageBackground = wrapContext(reactNative.ImageBackground);
const KeyboardAvoidingView = wrapContext(reactNative.KeyboardAvoidingView);
const Modal = wrapContext(reactNative.Modal);
const Pressable = wrapContext(reactNative.Pressable);
const ScrollView = wrapContext(reactNative.ScrollView);
const SectionList = wrapContext(reactNative.SectionList);
const Text = wrapContext(reactNative.Text);
const TouchableHighlight = wrapContext(reactNative.TouchableHighlight);
const TouchableOpacity = wrapContext(reactNative.TouchableOpacity);
const TouchableWithoutFeedback = wrapContext(reactNative.TouchableWithoutFeedback);
const SafeAreaView = wrapContext(reactNative.SafeAreaView);
const View = wrapContext(reactNative.View);
const VirtualizedList = wrapContext(reactNative.VirtualizedList);

Object.defineProperty(exports, 'ActivityIndicator', {
    enumerable: true,
    get: function () { return reactNative.ActivityIndicator; }
});
Object.defineProperty(exports, 'Button', {
    enumerable: true,
    get: function () { return reactNative.Button; }
});
Object.defineProperty(exports, 'DrawerLayoutAndroid', {
    enumerable: true,
    get: function () { return reactNative.DrawerLayoutAndroid; }
});
Object.defineProperty(exports, 'Image', {
    enumerable: true,
    get: function () { return reactNative.Image; }
});
Object.defineProperty(exports, 'InputAccessoryView', {
    enumerable: true,
    get: function () { return reactNative.InputAccessoryView; }
});
Object.defineProperty(exports, 'RefreshControl', {
    enumerable: true,
    get: function () { return reactNative.RefreshControl; }
});
Object.defineProperty(exports, 'TouchableNativeFeedback', {
    enumerable: true,
    get: function () { return reactNative.TouchableNativeFeedback; }
});
exports.FlatList = FlatList;
exports.ImageBackground = ImageBackground;
exports.KeyboardAvoidingView = KeyboardAvoidingView;
exports.Modal = Modal;
exports.Pressable = Pressable;
exports.SafeAreaView = SafeAreaView;
exports.ScrollView = ScrollView;
exports.SectionList = SectionList;
exports.Text = Text;
exports.TouchableHighlight = TouchableHighlight;
exports.TouchableOpacity = TouchableOpacity;
exports.TouchableWithoutFeedback = TouchableWithoutFeedback;
exports.View = View;
exports.VirtualizedList = VirtualizedList;
//# sourceMappingURL=index.js.map
